"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LcovParser = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const parse = require("lcov-parse");
class LcovParser {
    constructor(path) {
        if (!path || !(typeof path === 'string')) {
            throw new Error('请传递 string');
        }
        if (!fs_extra_1.default.existsSync(path) || !fs_extra_1.default.statSync(path).isFile()) {
            throw new Error('文件不存在');
        }
        this.path = path;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.parserLcov();
            return this.format();
        });
    }
    parserLcov() {
        return __awaiter(this, void 0, void 0, function* () {
            this.parserRes = yield new Promise((resolve, reject) => {
                parse(this.path, (err, data) => {
                    if (err) {
                        reject(err);
                    }
                    resolve(data);
                });
            });
        });
    }
    format() {
        const lcov = { detail: {} };
        let found = 0;
        let hit = 0;
        this.parserRes.forEach((item) => {
            found += item.lines.found;
            hit += item.lines.hit;
            const temp = {
                lineRate: item.lines.hit / item.lines.found,
                lines: [],
            };
            item.lines.details.forEach((detail) => {
                temp.lines.push({
                    number: detail.line,
                    hits: detail.hit,
                    branch: '',
                });
            });
            lcov.detail[item.file] = temp;
        });
        lcov.$ = {
            linesValid: found,
            linesCovered: hit,
        };
        return lcov;
    }
}
exports.LcovParser = LcovParser;
